@@ -3,13 +3,15 @@ import time
 from abc import abstractmethod
 from functools import partial
 from threading import Thread
-from typing import Iterable, NoReturn, Optional, Self, Union
+from typing import Iterable, NoReturn, Optional, Union
 
 from .cbindings import CBindings
-from .error import VMError
+from .error import CAPIError, VMError
+from .inst import bits
 from .kinds import KindId
+from .misc import Event, Midi
 from .subject import Subject
-from .util import polling, script
+from .util import comp, grouper, polling, script
 
 
 class Remote(CBindings):
@@ -18,15 +20,19 @@ class Remote(CBindings):
     DELAY = 0.001
 
     def __init__(self, **kwargs):
-        self.cache = {}
-        self.subject = Subject()
         self.strip_mode = 0
+        self.cache = {}
+        self.cache["strip_level"], self.cache["bus_level"] = self._get_levels()
+        self.midi = Midi()
+        self.subject = Subject()
         self.running = None
 
         for attr, val in kwargs.items():
             setattr(self, attr, val)
 
-    def __enter__(self) -> Self:
+        self.event = Event(self.subs)
+
+    def __enter__(self):
         """setup procedures"""
         self.login()
         self.init_thread()
@@ -40,38 +46,52 @@ class Remote(CBindings):
     def init_thread(self):
         """Starts updates thread."""
         self.running = True
+        print(f"Listening for {', '.join(self.event.get())} events")
         t = Thread(target=self._updates, daemon=True)
         t.start()
 
     def _updates(self):
-        """Continously update observers of dirty states."""
-        self.cache["strip_level"], self.cache["bus_level"] = self._get_levels()
+        """
+        Continously update observers of dirty states.
 
+        Generate _strip_comp, _bus_comp and update level cache if ldirty.
+
+        Runs updates at a rate of self.ratelimit.
+        """
         while self.running:
-            if self.pdirty:
+            if self.event.pdirty and self.pdirty:
                 self.subject.notify("pdirty")
-            if self.mdirty:
+            if self.event.mdirty and self.mdirty:
                 self.subject.notify("mdirty")
-            if self.ldirty:
+            if self.event.midi and self.get_midi_message():
+                self.subject.notify("midi")
+            if self.event.ldirty and self.ldirty:
+                self._strip_comp, self._bus_comp = (
+                    tuple(
+                        not x for x in comp(self.cache["strip_level"], self._strip_buf)
+                    ),
+                    tuple(not x for x in comp(self.cache["bus_level"], self._bus_buf)),
+                )
                 self.cache["strip_level"] = self._strip_buf
                 self.cache["bus_level"] = self._bus_buf
                 self.subject.notify("ldirty")
 
-            time.sleep(self.ratelimit)
+            time.sleep(self.ratelimit if self.event.any() else 0.5)
 
     def login(self) -> NoReturn:
         """Login to the API, initialize dirty parameters"""
         res = self.vm_login()
-        if res == 0:
-            print(f"Successfully logged into {self}")
-        elif res == 1:
+        if res == 1:
             self.run_voicemeeter(self.kind.name)
+        elif res != 0:
+            raise CAPIError(f"VBVMR_Login returned {res}")
+        print(f"Successfully logged into {self}")
         self.clear_dirty()
 
     def run_voicemeeter(self, kind_id: str) -> NoReturn:
         if kind_id not in (kind.name.lower() for kind in KindId):
             raise VMError(f"Unexpected Voicemeeter type: '{kind_id}'")
-        if kind_id == "potato" and ct.sizeof(ct.c_voidp) == 8:
+        if kind_id == "potato" and bits == 8:
             value = KindId[kind_id.upper()].value + 3
         else:
             value = KindId[kind_id.upper()].value
@@ -90,11 +110,12 @@ class Remote(CBindings):
         """Returns Voicemeeter's version as a string"""
         ver = ct.c_long()
         self.vm_get_version(ct.byref(ver))
-        v1 = (ver.value & 0xFF000000) >> 24
-        v2 = (ver.value & 0x00FF0000) >> 16
-        v3 = (ver.value & 0x0000FF00) >> 8
-        v4 = ver.value & 0x000000FF
-        return f"{v1}.{v2}.{v3}.{v4}"
+        return "{}.{}.{}.{}".format(
+            (ver.value & 0xFF000000) >> 24,
+            (ver.value & 0x00FF0000) >> 16,
+            (ver.value & 0x0000FF00) >> 8,
+            ver.value & 0x000000FF,
+        )
 
     @property
     def pdirty(self) -> bool:
@@ -110,16 +131,10 @@ class Remote(CBindings):
     def ldirty(self) -> bool:
         """True iff levels have been updated."""
         self._strip_buf, self._bus_buf = self._get_levels()
-        self._strip_comp, self._bus_comp = (
-            tuple(
-                not a == b
-                for a, b in zip(self.cache.get("strip_level"), self._strip_buf)
-            ),
-            tuple(
-                not a == b for a, b in zip(self.cache.get("bus_level"), self._bus_buf)
-            ),
+        return not (
+            self.cache.get("strip_level") == self._strip_buf
+            and self.cache.get("bus_level") == self._bus_buf
         )
-        return any(any(l) for l in (self._strip_comp, self._bus_comp))
 
     def clear_dirty(self):
         while self.pdirty or self.mdirty:
@@ -222,6 +237,24 @@ class Remote(CBindings):
             ),
         )
 
+    def get_midi_message(self):
+        n = ct.c_long(1024)
+        buf = ct.create_string_buffer(1024)
+        res = self.vm_get_midi_message(ct.byref(buf), n)
+        if res > 0:
+            vals = tuple(
+                grouper(3, (int.from_bytes(buf[i], "little") for i in range(res)))
+            )
+            for msg in vals:
+                ch, pitch, vel = msg
+                if not self.midi._channel or self.midi._channel != ch:
+                    self.midi._channel = ch
+                self.midi._most_recent = pitch
+                self.midi._set(pitch, vel)
+            return True
+        elif res == -1 or res == -2:
+            raise CAPIError(f"VBVMR_GetMidiMessage returned {res}")
+
     @script
     def sendtext(self, script: str):
         """Sets many parameters from a script"""
@@ -244,8 +277,7 @@ class Remote(CBindings):
                 return getattr(self, obj)[index]
             elif obj == "vban":
                 return getattr(getattr(self, obj), f"{m2}stream")[index]
-            else:
-                raise ValueError(obj)
+            raise ValueError(obj)
 
         [param(key).apply(datum).then_wait() for key, datum in data.items()]
 
@@ -266,8 +298,9 @@ class Remote(CBindings):
         self.clear_dirty()
         time.sleep(0.1)
         res = self.vm_logout()
-        if res == 0:
-            print(f"Successfully logged out of {self}")
+        if res != 0:
+            raise CAPIError(f"VBVMR_Logout returned {res}")
+        print(f"Successfully logged out of {self}")
 
     def end_thread(self):
         self.running = False
